/***********************************************************************
ColorSafe 1.0 Beta - http://www.woven-media.com/

A Java 6 Swing application for color testing and conversions from RGB to
hexadecimal data formats and vice versa.

Copyright (C) 2008-2009, Brent Allen Parrish <info@woven-media.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
************************************************************************/
package com.woven_media.colorsafe;

// woven-media.com
import com.woven_media.colorsafe.WindowUtil;
import com.woven_media.colorsafe.Application;

// util
import java.util.Locale;
// awt
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Window;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
// Swingx
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JPanel;

/**
 * Driver for <code>ColorSafe 1.0</code> application.
 *
 * @author  Brent Allen Parrish
 * @version %I%, %G%
 * @since   1.0
 */
public abstract class App extends Application
{
    public static final String TITLE_APP = "ColorSafe 1.0 Beta";
    public static final String TITLE_URL = "woven-media.com";
    public static final String ICON_IMG  = "colorsafeicon.gif";
    public static final String ABOUT_IMG = "about_green.jpg";
    public static final Color  BG_COLOR  = new Color(200, 200, 200); // lt blue 184, 207, 229
    public static final Locale LOCALE    = new Locale("en", "US");

    /**
     * Application entry point that implements the <code>java.lang.Runnable</code> interface as
     * an anonymous class. The anonymous <code>Runnable</code> instance is passed to the static
     * method <code>java.awt.EventQueue.invokeLater</code> method.
     *
     * @param args a string array of commandline arguments.
     * @see        java.lang.Runnable
     * @see        java.awt.EventQueue
     * @see        java.awt.EventQueue#invokeLater
     * @see        WindowUtil
     * @since      1.0
     */
    public static void main(String[] args)
    {

        // Schedules a job for the dispatch thread.
        EventQueue.invokeLater(new Runnable()
        {

            /**
             * Starts the application.
             * Required method of the <code>java.lang.Runnable</code> interface
             * that allows the application to be run in a seperate thread.
             *
             * @since 1.0
             */
            public void run()
            {
                // Sets up LAF
                JDialog.setDefaultLookAndFeelDecorated(true);
                //JFrame.setDefaultLookAndFeelDecorated(true);
                // App window setup
                JFrame f;
                f = new JFrame(TITLE_APP + " - " + TITLE_URL);
                f.setResizable(false);
                f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

                // ColorSafe Controller config
                Controller.setLocale(LOCALE);
                Controller.setBgColor(BG_COLOR);
                Controller controller = new Controller(f);

                // Centers window location within the viewing screen
                Dimension center = WindowUtil.centerWindow(null, f);
                f.setLocation(center.width, center.height);

                // Shows the app
                f.setVisible(true);
            }
        }); // End Runnable
    }
} // End class
