/**********************************************************************
ColorSafe 1.0 Beta - http://www.woven-media.com/

A Java 6 Swing application for color testing and conversions from RGB to
hexadecimal data formats and vice versa.

Copyright (C) 2008-2009, Brent Allen Parrish <info@woven-media.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
************************************************************************/
package com.woven_media.colorsafe;

// woven-media.com
import com.woven_media.colorsafe.ColorUtil;
// awt
import java.awt.Font;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Container;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.text.NumberFormat;
import java.lang.String;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;

/**
 * ColorInfoPanel extends <code>javax.swing.JPanel</code> and provides the safe color information panel
 * for the <ColorSafe> GUI.
 *
 * @author  Brent Allen Parrish
 * @version %I%, %G%
 * @since   1.0
 */

public class ColorInfoPanel extends JPanel
{
    private static ResourceBundle rb;
    private Font f = new Font("Monospaced", Font.PLAIN, 11);
    private Color c = new Color(0, 0, 0);
    private int r, g, b;
    private String hex, red, green, blue;
    private JLabel redVal, bluVal, grnVal, hexVal, hueVal, satVal, brtVal, cynVal, magVal, yelVal, blkVal;
    private JLabel lblRed, lblGrn, lblBlu, lblHex, lblHue, lblSat, lblbrt, lblCyn, lblMag, lblYel, lblBlk;
    private JLabel[] labels;
    private JLabel[] values;
    private GridBagLayout gb;
    private GridLayout gl;
    private Container cp;
    private JFrame jf;
    private Controller cntlr;

    /**
     * ColorInfoPanel class constructor receives <code>javax.swing.JFrame</code> reference, instantiates
     * all components and sets up UI layout.
     *
     * @param frame parent JFrame reference
     */
    ColorInfoPanel(JFrame frame, Controller controller)
    {
        jf = frame;
        cntlr = controller;
        rb = Controller.getResources();
        setBackground(Controller.getBgColor());
        lblRed = new JLabel(rb.getString("rLabel") + ": ", JLabel.RIGHT); // I18N
        lblGrn = new JLabel(rb.getString("gLabel") + ": ", JLabel.RIGHT); // I18N
        lblBlu = new JLabel(rb.getString("bLabel") + ": ", JLabel.RIGHT); // I18N
        lblHex = new JLabel(rb.getString("hLabel") + ": ", JLabel.RIGHT); // I18N
        lblCyn = new JLabel("C:", JLabel.RIGHT);
        lblMag = new JLabel("M:", JLabel.RIGHT);
        lblYel = new JLabel("Y:", JLabel.RIGHT);
        lblBlk = new JLabel("K:", JLabel.RIGHT);
        lblHue = new JLabel("H:", JLabel.RIGHT);
        lblSat = new JLabel("S:", JLabel.RIGHT);
        lblbrt = new JLabel("B:", JLabel.RIGHT);
        redVal = new JLabel("", JLabel.CENTER);
        bluVal = new JLabel("", JLabel.CENTER);
        grnVal = new JLabel("", JLabel.CENTER);
        hexVal = new JLabel("", JLabel.CENTER);
        hueVal = new JLabel("", JLabel.CENTER);
        satVal = new JLabel("", JLabel.CENTER);
        brtVal = new JLabel("", JLabel.CENTER);
        cynVal = new JLabel("", JLabel.CENTER);
        magVal = new JLabel("", JLabel.CENTER);
        yelVal = new JLabel("", JLabel.CENTER);
        blkVal = new JLabel("", JLabel.CENTER);
        labels = new JLabel[] {lblRed, lblGrn, lblBlu, lblHex, lblCyn, lblMag, lblYel, lblBlk, lblHue, lblSat, lblbrt};
        values = new JLabel[] {redVal, bluVal, grnVal, hexVal, cynVal, magVal, yelVal, blkVal, hueVal, satVal, brtVal};

        gl = new GridLayout(4, 1, 2, 2);
        gb = new GridBagLayout();
        setLayout(gb);

        setPreferredSize(new Dimension(320, 100));
        setDoubleBuffered(true);

        for(int i=0; i < labels.length; i++)
        {
            labels[i].setForeground(new Color(102, 102, 153));
            labels[i].setFont(new Font("Monospaced", Font.BOLD, 11));
            labels[i].setPreferredSize(new Dimension(50, 20));
            values[i].setForeground(Color.black);
            values[i].setFont(new Font("Monospaced", Font.PLAIN, 11));
            values[i].setPreferredSize(new Dimension(50, 20));
        }

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.weightx = 0.5;

        JPanel col1 = new JPanel(gl);
        col1.add(labels[0]);
        col1.add(labels[1]);
        col1.add(labels[2]);
        col1.add(labels[3]);

        gbc.anchor = GridBagConstraints.CENTER;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridheight = 1; gbc.gridwidth = 1;
        gbc.gridx = 0; gbc.gridy = 0;
        add(col1, gbc);

        JPanel col2 = new JPanel(gl);
        col2.add(values[0]);
        col2.add(values[1]);
        col2.add(values[2]);
        col2.add(values[3]);

        gbc.anchor = GridBagConstraints.CENTER;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridheight = 1; gbc.gridwidth = 1;
        gbc.gridx = 1; gbc.gridy = 0;
        add(col2, gbc);

        JPanel col3 = new JPanel(gl);
        col3.add(labels[4]);
        col3.add(labels[5]);
        col3.add(labels[6]);
        col3.add(labels[7]);

        gbc.anchor = GridBagConstraints.CENTER;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridx = 2; gbc.gridy = 0;
        gbc.gridheight = 1; gbc.gridwidth = 1;
        add(col3, gbc);

        JPanel col4 = new JPanel(gl);
        col4.add(values[4]);
        col4.add(values[5]);
        col4.add(values[6]);
        col4.add(values[7]);

        gbc.anchor = GridBagConstraints.CENTER;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridx = 3; gbc.gridy = 0;
        gbc.gridheight = 1; gbc.gridwidth = 1;
        add(col4, gbc);

        JPanel col5 = new JPanel(gl);
        col5.setBounds(10, 10, 42, 90);
        col5.add(labels[8]);
        col5.add(labels[9]);
        col5.add(labels[10]);

        gbc.anchor = GridBagConstraints.CENTER;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridx = 4; gbc.gridy = 0;
        gbc.gridheight = 1; gbc.gridwidth = 1;
        add(col5, gbc);

        JPanel col6 = new JPanel(gl);
        col6.add(values[8]);
        col6.add(values[9]);
        col6.add(values[10]);

        gbc.anchor = GridBagConstraints.CENTER;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridheight = 1; gbc.gridwidth = 1;
        gbc.gridx = 5; gbc.gridy = 0;
        add(col6, gbc);
    }

    /**
     * Stores current RGB and Hexadecimal color representations for this instance.
     *
     * @param r   <code>int</code> value between 0 - 255.
     * @param g   <code>int</code> value between 0 - 255.
     * @param b   <code>int</code> value between 0 - 255.
     * @param hex hexdecimal triplet number between 000000 - FFFFFF.
     * @since 1.0
     */
    public void colorInformation(int r, int g, int b, String hex)
    {
        this.r = r;
        this.g = g;
        this.b = b;
        this.hex = hex;
    }

    /**
     * Updates and formats text for the <code>ColorInfoPanel</code> current selected color information.
     *
     * @since 1.0
     */
    public void updateLabels()
    {
        String red, grn, blu;
        String [] hsb = ColorUtil.getHSBvalues(this.r, this.g, this.b);
        int[] rtc = ColorUtil.rgbToCMYK(this.r, this.g, this.b);
        double[] cmyk = ColorUtil.cmykAsPercent(rtc[0], rtc[1], rtc[2], rtc[3]);
        double[] rgbArr = ColorUtil.hexToFloatArray(this.hex);
        int[] ints = ColorUtil.rgbFloatToUInt(rgbArr[0], rgbArr[1], rgbArr[2]);
        NumberFormat f = NumberFormat.getInstance();
        f.setMaximumFractionDigits(0);

        if(cntlr.isArithmeticConversion())
        {
            red = ints[0] + "";
            grn = ints[1] + "";
            blu = ints[2] + "";
        }
        else
        {
            red = rgbArr[0] + ""; //this.r + "";
            grn = rgbArr[1] + ""; //this.g + "";
            blu = rgbArr[2] + ""; //this.b + "";
        }

        values[0].setText(red);
        values[1].setText(grn);
        values[2].setText(blu);
        values[3].setText("#" + this.hex);
        values[4].setText(f.format(cmyk[0]) + "%");
        values[5].setText(f.format(cmyk[1]) + "%");
        values[6].setText(f.format(cmyk[2]) + "%");
        values[7].setText(f.format(cmyk[3]) + "%");
        values[8].setText(hsb[0]);
        values[9].setText(hsb[1]);
        values[10].setText(hsb[2]);
    }

    /**
     * Overridden <code>paintComponent</code> method which passes the <code>java.awt.Graphics</code> object to
     * the <code>javax.swing.JPanel</code> constructor so that the background can be repainted to display
     * color data in the panel dynamically.
     *
     * @param g <code>java.awt.Graphics</code> object passed to the <code>super</code> class.
     * @since   1.0
     */
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);  //paint background
        updateLabels();
    }

}
