/***********************************************************************
ColorSafe 1.0 Beta - http://www.woven-media.com/

A Java 6 Swing application for color testing and conversions from RGB to
hexadecimal data formats and vice versa.

Copyright (C) 2008-2009, Brent Allen Parrish <info@woven-media.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
************************************************************************/
package com.woven_media.colorsafe;

// woven-media.com
import com.woven_media.colorsafe.IPalette;
import com.woven_media.colorsafe.CutNPastePopup;
import com.woven_media.colorsafe.NewFocusTraversalPolicy;
import com.woven_media.colorsafe.WindowUtil;
import com.woven_media.colorsafe.ColorUtil;
import com.woven_media.colorsafe.About;
// io
import java.io.IOException;
// net
import java.net.URL;
// util
import java.util.Locale;
import java.util.Vector;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
// awt
import java.awt.Container;
import java.awt.Color;
import java.awt.Font;
import java.awt.Label;
import java.awt.Point;
import java.awt.Insets;
import java.awt.Panel;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.FocusTraversalPolicy;
// awt.datatransfer
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.DataFlavor;
// awt.event
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.InputEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.MouseAdapter;
// swingx
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JScrollBar;
import javax.swing.JComponent;
import javax.swing.JSlider;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.ImageIcon;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JColorChooser;
import javax.swing.colorchooser.ColorSelectionModel;
import javax.swing.colorchooser.AbstractColorChooserPanel;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeListener;
import javax.swing.JOptionPane;
import javax.swing.BorderFactory;
import javax.swing.JRootPane;
import javax.swing.JMenuBar;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.MenuElement;
import javax.swing.MenuSelectionManager;
import javax.swing.JMenu;

/**
 * Controller class handles UI and event-handling.
 *
 * @author  Brent Allen Parrish
 * @version %I%, %G%
 * @since   1.0
 */
public class Controller implements ChangeListener, IPalette
{
    /** Constants **/
    // Default resources path
    public static final String DEF_RES_PATH = "resources/default/";
    // Default swatch color
    public static final Color DEFAULT_SWATCH = Color.BLACK;
    // Maxlength for text fields
    public static final int TXT_MAXLENGTH = 6;

    /** Private Members **/
    // Static
    private static JFrame host;
    // Swingx
    private JPanel colorSwatch, colorSafeSwatch;
    private JPanel lPanel, levelPanel, bigUpperPanel;
    private JPanel swatchPanel, colorInfoPanel;
    private JButton cancelBtn, resetBtn, applyBtn;
    private JTextField fldRed, fldGreen, fldBlue, fldHex;
    private JSlider sliderRed, sliderGrn, sliderBlu, sliderEmp;
    private JLabel rLabel, gLabel, bLabel, hLabel, swatchLabel, safeSwatchLabel;
    private TitledBorder panelBorder, infoBorder;
    private JPopupMenu popup;
    private JCheckBox chkBox;
    private JTextField[] jFields;
    private JSlider[] jSliders;
    private JLabel[] jLabels;
    private JMenuItem[] mi;
    // Primitives
    private double num2;
    private int controlsLength, num;
    private boolean hexCalc, init;
    private String fieldValue;
    // User-defined
    private ColorUtil hex;
    private ColorInfoPanel info;
    // UI
    private Font bf;
    private Border etched;
    // References
    private Container cp;
    private Vector<Component> order;
    // Static
    private static ResourceBundle rb;
    private static Locale locale;
    private static URL url;
    private static Color c, bgColor, infoColor, newColor, currSel, curSafe, swatchDefault;
    private static ImageIcon iconImg, flagUS, flagDE;

    /**
     * Class constructor sets up the <code>javax.swing.JFrame</code> and calls
     * <code>Controller.start</code>. The constructor sets an <code>init</code> flag
     * to <code>true</code> indicating that this class is being initialized, then sets
     * background color and loads the image icon.
     *
     * @param f <code>javax.swing.JFrame</code> reference to the parent window.
     * @since   1.0
     */
    public Controller(JFrame f)
    {
         init = true;
         host = f;
         cp = host.getContentPane();
         cp.setBackground(bgColor);
         loadIcon(host);
         start();
    }

    /**
     * Starts the application.
     * The <code>start</code> method grabs the resource bundle and sets up the UI, if the
     * <code>init</code> flag is false, the <code>start</code> method is
     * not being called by the constructor and the <code>this.removeComponents</code> method is
     * called to reload the components in the content pane. This behavior is required for
     * toggling language options at runtime, since the components need to be reloaded with
     * new string values from the resource bundle's <code>strings_xx.properties</code> file.
     * The <code>xx</code> suffix specifies the standardized two-letter lowercase language code.
     * <p>
     * <h4>e.g.</h4>
     * <dl>
     * <dt>English</dt>
     * <dd>strings_en.properties</dd>
     * <dt>German</dt>
     * <dd>strings_de.properties</dd>
     * <dt>French</dt>
     * <dd>strings_fr.properties</dd>
     * </dl>
     * ...etc.
     * </p>
     * Adds the menubar and all the UI components. Calls <code>resetAction</code>
     * to setup default color settings.
     * <p>
     * NOTE: The <code>host</code> reference needs to call the <code>javax.swing.JFrame#pack</code>
     * method to set the preferred size and the layouts of its subcomponents.
     * </p>
     *
     * @since 1.0
     */
    private void start()
    {
        getResources();
        ColorSafeUI.setController(this);
        ColorSafeUI.setUI();
        if(!init)
        {
            removeComponents();
        }
        createMenu(host);
        createComponents();
        createUI();
        ColorSafeUI.updateComponentUI(host);
        host.pack(); // <<< IMPORTANT!
        init = false;
    }

    /**
     * Calls <code>Controller.start</code>.
     * The <code>restart</code> method <i>reloads</i> this app instance without destroying
     * the enclosing parent frame. Convenience method only. Used by the <code>ColorSafeMenubar</code>
     * class to handle the language toggle event at runtime.
     *
     * @since 1.0
     */
    public void restart()
    {
        start();
    }

    /**
     * Removes <code>javax.swing.JPanels</code> from the content pane.
     *
     * @see   javax.swing.JPanel#remove
     * @since 1.0
     */
    private void removeComponents()
    {
        Component[] components = host.getContentPane().getComponents();
        //Component[] comps = host.getLayeredPane().getComponents();
        for(int i = 0; i < components.length; ++i)
        {
            if(components[i] instanceof JPanel)
            {
                host.remove(components[i]);
            }
        }
        host.repaint();
    }

    /**
     * Sets this app's background color.
     *
     * @param c <code>java.awt.Color</code> object
     * @since   1.0
     */
    public static void setBgColor(Color c)
    {
        bgColor = c;
    }

    /**
     * Returns this app's background color.
     *
     * @return <code>java.awt.Color</code> object of the background color
     *         for this instance.
     * @since  1.0
     */
    public static Color getBgColor()
    {
        return bgColor;
    }


    /**
     * Returns this instance's icon image.
     *
     * @return iconImg <code>javax.swing.IconImage</code>
     */
    public static ImageIcon getImageIcon()
    {
        return iconImg;
    }

    /**
     * Returns the swatch panel's current selected color.
     *
     * @return <code>java.awt.Color</code> object of this swatch panel's background color
     *         if <code>colorSwatch</code> panel is not <code>null</code>, otherwise
     *         returns <code>null</code>.
     * @since  1.0
     */
    public Color getCurrentSelectedColor()
    {
        if(colorSwatch != null)
        {
            return colorSwatch.getBackground();
        }
        else
        {
            return null;
        }
    }

    /**
     * Returns the swatch panel's current selected safe color.
     *
     * @return <code>java.awt.Color</code> object of this safe swatch panel's background
     *         color if <code>colorSafeSwatch</code> panel is not <code>null</code>, otherwise
     *         returns <code>null</code>.
     * @since  1.0
     */
    public Color getCurrentSafeColor()
    {
        if(colorSafeSwatch != null)
        {
            return colorSafeSwatch.getBackground();
        }
        else
        {
            return null;
        }
    }

    /**
     * Sets <code>Controller.locale</code>. The <code>locale</code> setting is primarily used for
     * multiple language support.
     *
     * @param loc a <code>java.util.Locale</code> object primarily used for setting language options.
     * @see   java.util.Locale
     * @since 1.0
     */
    public static void setLocale(Locale loc)
    {
        if((loc != null) && (loc != getLocale()))
        {
            locale = loc;
        }
    }

    /**
     * Returns the <code>Controller.locale</code> member.
     *
     * @return <code>java.util.Locale</code> object
     * @see    java.util.Locale
     * @since  1.0
     */
    public static Locale getLocale()
    {
        return locale;
    }

    /**
     * Reads in the <code>java.util.ResourceBundle <i>strings_xx.properties</i></code> file
     * used by this GUI instance.
     *
     * @return <code>java.util.ResourceBundle</code> used for localization support.
     * @see    java.util.ResourceBundle
     * @since  1.0
     */
    public static ResourceBundle getResources()
    {
        // Default strings_en.properties file
        String resString = "resources.strings_en";
        try
        {
            // TODO: Create a map to avoid a bunch of string comps
            if(locale.getLanguage() == "de")
            {
                resString = "resources.strings_de";
            }
            rb = ResourceBundle.getBundle(resString, locale);
            return rb;
        }
        catch(MissingResourceException mre)
        {
            // TODO: improve error handling!
            mre.printStackTrace();
            return rb;
        }
    }

    /**
     * Sets the <code>Controller.url</code> member. Locates relative path resources.
     *
     * @param res <code>java.lang.String</code> resource.
     * @see       java.net.URLClassLoader
     * @since     1.0
     */
    public static void getURLResource(String res)
    {
        url = java.net.URLClassLoader.getSystemResource(res);
    }

    /**
     * Returns the <code>Controller.url</code> member.
     *
     * @return <code>java.net.URL</code> object.
     * @see    java.net.URL
     * @since  1.0
     */
    public URL getURL()
    {
        return url;
    }

    /**
     * Loads image icon from default resources.
     *
     * @param f parent <code>javax.swing.JFrame</code> reference.
     * @see     javax.swing.JFrame
     * @since   1.0
     */
    public static void loadIcon(JFrame f)
    {
        getURLResource(DEF_RES_PATH + App.ICON_IMG);
        iconImg = new ImageIcon(url);
        if(iconImg != null)
        {
            f.setIconImage(iconImg.getImage());
        }
    }

    /**
     * Creates the top-level menubar and specifies the host window.
     *
     * @param host parent <code>javax.swing.JFrame</code> reference.
     * @see        javax.swing.JMenuBar
     * @see        ColorSafeMenuBar
     * @since      1.0
     */
    private void createMenu(JFrame host)
    {
        JMenuBar menu = (JMenuBar)new ColorSafeMenuBar(this, this.host);
    }

    /**
     * Instantiates components for the content pane; includes buttons,
     * sliders, panels, labels, text fields, popup and borders.
     *
     * @since 1.0
     */
    private void createComponents()
    {
        controlsLength = 4;
        //maxLength = 6; // maxlength for text fields
        //swatchDefault = Color.BLACK;
        hexCalc = false;
        infoColor = new Color(51, 51, 51);
        bf = new Font("Arial", Font.PLAIN, 10);
        // cancel
        cancelBtn = new JButton(rb.getString("cancelBtn")); // I18N
          cancelBtn.setMnemonic(KeyEvent.VK_C);
        // reset
        resetBtn = new JButton(rb.getString("resetBtn"));   // I18N
        // apply
        applyBtn = new JButton(rb.getString("applyBtn"));   // I18N
        // JTextField
        fldRed = new JTextField("0");
        fldGreen = new JTextField("0");
        fldBlue = new JTextField("0");
        fldHex = new JTextField("000000");
        chkBox = new JCheckBox("Arithmetic");
        jFields = new JTextField[] {fldRed, fldGreen, fldBlue, fldHex};
        // JSlider
        sliderRed = new JSlider(JScrollBar.HORIZONTAL, 0, 255, 0);
          sliderRed.setToolTipText(rb.getString("slider.red"));        // I18N
        sliderGrn = new JSlider(JSlider.HORIZONTAL, 0, 255, 0);
          sliderGrn.setToolTipText(rb.getString("slider.grn"));        // I18N
        sliderBlu = new JSlider(JSlider.HORIZONTAL, 0, 255, 0);
          sliderBlu.setToolTipText(rb.getString("slider.blu"));        // I18N
        sliderEmp = new JSlider(JSlider.HORIZONTAL, 0, 255, 0);
        jSliders = new JSlider[] {sliderRed, sliderGrn, sliderBlu, sliderEmp};
        // JPanel
        swatchPanel = new JPanel();
        colorInfoPanel = new JPanel();
        colorSwatch = new JPanel();
        colorSafeSwatch = new JPanel();
        info = new ColorInfoPanel(host, this);
        // JLabel
        rLabel = new JLabel(rb.getString("rLabel") + ": "); // I18N
        gLabel = new JLabel(rb.getString("gLabel") + ": "); // I18N
        bLabel = new JLabel(rb.getString("bLabel") + ": "); // I18N
        hLabel = new JLabel(rb.getString("hLabel") + ": "); // I18N
        swatchLabel = new JLabel(rb.getString("swatchLabel"), Label.LEFT); // I18N
        safeSwatchLabel = new JLabel(rb.getString("safeSwatchLabel"), Label.LEFT); // I18N
        jLabels = new JLabel[] {rLabel, gLabel, bLabel, hLabel};
        // Borders
        etched = BorderFactory.createEtchedBorder();
        panelBorder = BorderFactory.createTitledBorder(etched, rb.getString("etchedBorder")); // I18N
        infoBorder = BorderFactory.createTitledBorder(etched,
                                                      rb.getString("infoBorder"),
                                                      TitledBorder.DEFAULT_JUSTIFICATION,
                                                      TitledBorder.DEFAULT_POSITION,
                                                      bf,
                                                      infoColor); // I18N
    }

    /**
     * Sets and lays out the instantiated components for the GUI.
     *
     * @since 1.0
     */
    private void createUI()
    {
        Dimension panelSize = new Dimension(200, 110);

        colorSwatch.setPreferredSize(panelSize);
        colorSafeSwatch.setPreferredSize(panelSize);

        swatchLabel.setFont(bf);
        safeSwatchLabel.setFont(bf);
        safeSwatchLabel.setSize(200, 20);

        swatchPanel.setBorder(panelBorder);
        swatchPanel.getInsets();
        swatchPanel.setLayout(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0; gbc.gridy = 0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        swatchPanel.add(swatchLabel, gbc);
        gbc.gridx = 0; gbc.gridy = 1;
        swatchPanel.add(colorSwatch, gbc);
        gbc.gridx = 0; gbc.gridy = 2;
        swatchPanel.add(colorSafeSwatch, gbc);
        gbc.gridx = 0; gbc.gridy = 3;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        swatchPanel.add(safeSwatchLabel, gbc);

        // Build upper panel with 4 horz. rows.
        bigUpperPanel = new JPanel();
        bigUpperPanel.setLayout(new GridLayout(1, 3, 20, 0));

        Dimension sliderW = new Dimension(sliderRed.getSize());
        Vector<Component> jFieldOrder = new Vector<Component>();

        for(int i = 0; i < controlsLength; ++i)
        {
            // Each row is a panel within bigUpperPanel.
            jSliders[i].setPreferredSize(sliderW);

            levelPanel = new JPanel();
            levelPanel.setLayout(new GridLayout(3, 1, 0, 10));
            levelPanel.add(jLabels[i]);
            levelPanel.add(jFields[i]);
            if(i == 3)
            {
                levelPanel.add(chkBox);
            }
            else
            {
                levelPanel.add(jSliders[i]);
            }

            bigUpperPanel.add(levelPanel);

            jFields[i].setForeground(Color.black);
            jFields[i].setFont(new Font("Arial", Font.BOLD, 12));
            jFieldOrder.add(jFields[i]);

            jLabels[i].setFont(new Font("SansSerif", Font.BOLD, 12));
            jLabels[i].setForeground(new Color(102, 102, 153));
            if(i == 3)
            {
                jSliders[i].setVisible(false);
            }
        }

        host.getContentPane().setLayout(new GridBagLayout());

        gbc.gridx = 0; gbc.gridy = 0;
        gbc.gridheight = 3;
        gbc.gridwidth = 1;
        gbc.insets = getInsets();
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.fill = GridBagConstraints.BOTH;

        host.getContentPane().add(swatchPanel, gbc);

        gbc.gridx = 1;  gbc.gridy = 0;
        gbc.gridheight = 1;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.SOUTH;

        host.getContentPane().add(bigUpperPanel, gbc);

        // Build lower panel containing buttons
        info.setBounds(0, 0, 280, 60);
        colorInfoPanel.setBorder(infoBorder);

        colorInfoPanel.add(info);

        gbc.gridx = 1;  gbc.gridy = 1;
        gbc.gridheight = 1;
        gbc.gridwidth = 1;
        gbc.fill = GridBagConstraints.BOTH;

        host.getContentPane().add(colorInfoPanel, gbc);

        JPanel lowerPanel = new JPanel();
        lowerPanel.add(applyBtn);
        lowerPanel.add(cancelBtn);
        lowerPanel.add(resetBtn);

        gbc.gridx = 1; gbc.gridy = 2;
        gbc.gridheight = 1;
        gbc.gridwidth = 1;

        host.getContentPane().add(lowerPanel, gbc);

        //applyBtn.setEnabled(false);
        applyBtn.setMnemonic(KeyEvent.VK_ENTER);
        //resetBtn.setEnabled(false);

        applyBtn.setToolTipText(rb.getString("tip.apply"));   // I18N
        resetBtn.setToolTipText(rb.getString("tip.reset"));   // I18N
        cancelBtn.setToolTipText(rb.getString("tip.cancel")); // I18N

        // JPopupMenus
        //hex
        JPopupMenu popx = new CutNPastePopup(host, fldHex);
        fldHex.setComponentPopupMenu(popx);
        //red
        JPopupMenu popr = new CutNPastePopup(host, fldRed);
        fldRed.setComponentPopupMenu(popr);
        //green
        JPopupMenu popg = new CutNPastePopup(host, fldGreen);
        fldGreen.setComponentPopupMenu(popg);
        //blue
        JPopupMenu popb = new CutNPastePopup(host, fldBlue);
        fldBlue.setComponentPopupMenu(popb);

        // Sets the FocusTraversalPolicy after the
        // focusable components have been created.
        host.setFocusTraversalPolicy(setTabOrder());

        // Event-handlers for components
        for(int i = 0; i < controlsLength; ++i)
        {

            jSliders[i].addChangeListener(this);
            jSliders[i].setValue(jSliders[i].getMaximum());

            final JTextField t = jFields[i];

            t.addKeyListener(new KeyAdapter()
            {
                public void keyPressed(KeyEvent ke)
                {
                    if(ke.getKeyCode() == KeyEvent.VK_ENTER)
                    {
                        applyAction();
                        setRGBcontrols();
                    }
                }
                public void keyTyped(KeyEvent ke)
                {
                    validateTextField(ke);
                }
            });
            t.addFocusListener(new FocusAdapter()
            {
                public void focusLost(FocusEvent fe)
                {
                    textFieldLostFocus(fe, t);
                    finalValidation(fe, t);
                }
            });
            t.addFocusListener(new FocusAdapter()
            {
                public void focusGained(FocusEvent fe)
                {
                    textFieldGainedFocus(fe, t);
                }
            });

        } // End For Loop
        // Arithmetic Check Box
        chkBox.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                setRGBcontrols();
                host.repaint();
            }
        });
        chkBox.addKeyListener(new KeyAdapter()
        {
            public void keyPressed(KeyEvent ke)
            {
                if(ke.getKeyCode() == KeyEvent.VK_ENTER)
                {
                    if(isArithmeticConversion())
                    {
                        chkBox.setSelected(false);
                    }
                    else
                    {
                        chkBox.setSelected(true);
                    }
                    applyAction();
                    setRGBcontrols();
                }
            }

        });
        // Apply
        applyBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                applyAction();
            }
        });
        applyBtn.addKeyListener(new KeyAdapter()
        {
            public void keyPressed(KeyEvent ke)
            {
                if(ke.getKeyCode() == KeyEvent.VK_ENTER)
                {
                    applyBtn.doClick();
                }
            }

        });
        // Reset
        resetBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                resetAction();
            }
        });
        resetBtn.addKeyListener(new KeyAdapter()
        {
            public void keyPressed(KeyEvent ke)
            {
                if(ke.getKeyCode() == KeyEvent.VK_ENTER)
                {
                    resetBtn.doClick();
                }
            }
        });
        // Cancel
        cancelBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                System.exit(0);
            }
        });
        cancelBtn.addKeyListener(new KeyAdapter()
        {
            public void keyPressed(KeyEvent ke)
            {
                if(ke.getKeyCode() == KeyEvent.VK_ENTER)
                {
                    cancelBtn.doClick();
                }
            }
        });
        // Sets default swatch color
        //Color swatchDefault = new Color(0, 0, 0);
        colorSwatch.setBackground(DEFAULT_SWATCH);
        colorSafeSwatch.setBackground(DEFAULT_SWATCH);
        setCurrentColor(DEFAULT_SWATCH);

    } // End createUI

    /**
     * Handles the focus cycle and tab order for this instance.
     *
     * @see   java.awt.FocusTraversalPolicy
     * @see   java.util.Vector
     * @see   NewFocusTraversalPolicy
     * @since 1.0
     */
    private FocusTraversalPolicy setTabOrder()
    {
        Vector<Component> order = new Vector<Component>(11);
        fldRed.setFocusCycleRoot(true);
        order.add(fldRed);
        order.add(fldGreen);
        order.add(fldBlue);
        order.add(fldHex);
        order.add(sliderRed);
        order.add(sliderGrn);
        order.add(sliderBlu);
        order.add(chkBox);
        order.add(applyBtn);
        order.add(cancelBtn);
        order.add(resetBtn);
        return new NewFocusTraversalPolicy(order);
    }

    /*
     * Indicates whether the two swatchs are both safe colors.
     *
     * @since 1.0
     */
    public boolean isSafeColor()
    {
        return ((colorSwatch.getBackground().equals(colorSafeSwatch.getBackground())));
    }

    /**
     * Sets selection for text entered into <code>javax.swing.JTextFields</code>.
     *
     * @param fe <code>java.awt.event.FocusEvent</code>
     * @see      javax.swing.JTextField
     * @see      java.awt.event.FocusEvent
     * @since    1.0
     */
    public void setSelection(FocusEvent fe)
    {
        for(int i = 0; i < controlsLength; ++i)
        {
            if(fe.getSource().equals(jFields[i]))
            {
                if(jFields[i].getText().length() > 0)
                {
                    jFields[i].selectAll();
                }
            }
        }
    }

    /**
     * Applies all changes made within the RGB and Hex text fields. Opens a warning message in
     * a dialog box if invalid input is detected.
     *
     * @since 1.0
     */
    public void applyAction()
    {
        if(!hexCalc)
        {

            if(isArithmeticConversion())
            {
                Double r, g, b;
                String red, grn, blu;
                red = (fldRed.getText().length() > 0) ? fldRed.getText() : "-1";
                grn = (fldGreen.getText().length() > 0) ? fldGreen.getText() : "-1";
                blu = (fldBlue.getText().length() > 0) ? fldBlue.getText() : "-1";
                try{r = Double.parseDouble(red);}catch(NumberFormatException nfe){r = 0.0;}
                try{g = Double.parseDouble(grn);}catch(NumberFormatException nfe){g = 0.0;}
                try{b = Double.parseDouble(blu);}catch(NumberFormatException nfe){b = 0.0;}
                r = ((red != "-1") && (r >= 0.0) && (r <= 1.0)) ? r : 0.0;
                g = ((grn != "-1") && (g >= 0.0) && (g <= 1.0)) ? g : 0.0;
                b = ((blu != "-1") && (b >= 0.0) && (b <= 1.0)) ? b : 0.0;
                int[] ints = ColorUtil.rgbFloatToUInt(r, g, b);
                newColor = new Color(ints[0], ints[1], ints[2]);
                colorSelection(1);
            }
            else
            {
                showColor(getRGBColor());
            }
        }
        else if((fldHex.getText().length() > 0) && ((fldHex.getText().length() % 3) == 0) && ColorUtil.isValidHex(fldHex.getText()))
        {
            showColor(ColorUtil.getRGBColor(fldHex.getText()));
            setRGBcontrols();
        }
        else
        {
            String title = rb.getString("error.title");                // I18N
            String msg = rb.getString("error.badhex");                 // I18N
            msgDialog(host, msg, title, JOptionPane.ERROR_MESSAGE);
            fldHex.selectAll();
        }
        showHexString();
    }

    /**
     * Resets this application back to its default values.
     *
     * @since 1.0
     */
    public void resetAction()
    {
        for(int i = 0; i < controlsLength; ++i)
        {
            if(i < (controlsLength - 1))
            {
                jFields[i].setText("255");
            }
            else
            {
                jFields[i].setText("FFFFFF");
            }
            jSliders[i].setValue(jSliders[i].getMinimum());
        }
        showSafeColor();
        setBgColor(App.BG_COLOR);
        cp.setBackground(App.BG_COLOR);
        restart();
    }

    /**
     * Handle actions when a <code>javax.swing.JtextField</code> gains focus.
     *
     * @param fe  <code>java.awt.event.FocusEvent</code>
     * @param txt <code>javax.swing.JtextField</code> reference of the field requesting focus.
     * @since     1.0
     */
    public void textFieldGainedFocus(FocusEvent fe, JTextField txt)
    {
        setSelection(fe);
        fieldValue = txt.getText();
        hexCalc = (fe.getSource().equals(fldHex)) ? true : false;
    }

    /**
     * Handles actions when a <code>javax.swing.JtextField</code> loses focus.
     *
     * @param fe  <code>java.awt.event.FocusEvent</code>
     * @param txt <code>javax.swing.JTextField</code>
     * @since     1.0
     */
    public void textFieldLostFocus(FocusEvent fe, JTextField txt)
    {
        return;
    }

    // Getters for private member fields

    public JTextField getRedField()
    {
        return fldRed;
    }

    public JTextField getGreenField()
    {
        return fldGreen;
    }

    public JTextField getBlueField()
    {
        return fldBlue;
    }

    /**
     * Determines selection of the Arithmetic conversion checkbox.
     *
     * @return boolean value.private JTextField fldRed, fldGreen, fldBlue, fldHex;
     * @since  1.0
     */
    public boolean isArithmeticConversion()
    {
        return chkBox.isSelected();
    }


    /**
     * Displays selected color for this instance.
     *
     * @param c <code>java.awt.Color</code> object.
     * @see     java.awt.Color
     * @since   1.0
     */
    public void showColor(Color c)
    {
        colorSwatch.setBackground(c);
    }

    /**
     * Reads the values from the RGB text fields and converts and returns <code>java.awt.Color</code> object
     * via the <code>ColorUtil</code> class.
     *
     * @return <code>java.awt.Color</code> object
     * @since  1.0
     */
    public Color getRGBColor()
    {
        Color rgb;
        String r, g, b;
        int[] ints = parseRGBfields();
        r = "" + ints[0];
        g = "" + ints[1];
        b = "" + ints[2];
        rgb = ColorUtil.getRGBColor(r, g, b);

        return rgb;
    }

    /**
     * Displays the current color information.
     *
     * @see   ColorUtil
     * @since 1.0
     */
    public void showSafeColor()
    {
        Color color;
        String hexValue = fldHex.getText();
        int[] ints = parseRGBfields();

        if(!hex.isColorSafe(hexValue))
        {
            int[] rgb = ColorUtil.buildRGBarray(hexValue);
            int[] arr = ColorUtil.nearestSafeColor(rgb);
            color = ColorUtil.getNearestSafeColor(rgb);
            colorSafeSwatch.setBackground(color);
            //Displays safe color info only
            //info.colorInformation(arr[0], arr[1], arr[2], ColorUtil.RGBtoHEX(arr[0], arr[1], arr[2]));
            info.colorInformation(ints[0], ints[1], ints[2], ColorUtil.RGBtoHEX(ints[0], ints[1], ints[2])); // Show all info
            info.repaint();
        }
        else
        {
            color = new Color(ints[0], ints[1], ints[2]);
            colorSafeSwatch.setBackground(color);
            info.colorInformation(ints[0], ints[1], ints[2], ColorUtil.RGBtoHEX(ints[0], ints[1], ints[2]));
            info.repaint();
        }
    }

    /**
     * Sets the hexadecimal value in the <code>javax.swing.JTextField</code>.
     *
     * @since 1.0
     */
    public void showHexString()
    {
        String hexString;
        int[] rgb = parseRGBfields();

        hexString = ColorUtil.RGBtoHEX(rgb[0], rgb[1], rgb[2]);
        fldHex.setText(hexString);
    }

    /**
     * Parses the RGB string textfield values for floating point or integer values.
     *
     * @since   1.0
     */
    public int[] parseRGBfields()
    {
        int r, g, b;
        int[] rgb = new int[3];

        if(isArithmeticConversion()) // Checks for floating point values
        {
            r = sliderRed.getValue();
            g = sliderGrn.getValue();
            b = sliderBlu.getValue();
        }
        else // Default decimal conversion
        {
            int red, grn, blu;
            try
            {
                red = Integer.parseInt(fldRed.getText());
                grn = Integer.parseInt(fldGreen.getText());
                blu = Integer.parseInt(fldBlue.getText());
            }
            catch(NumberFormatException nfe)
            {
                red = 0;
                grn = 0;
                blu = 0;
            }
            r = ((fldRed.getText().length() > 0) && (red >= 0) && (red <= 255)) ? red : 0;
            g = ((fldGreen.getText().length() > 0) && (grn >= 0) && (grn <= 255)) ? grn : 0;
            b = ((fldBlue.getText().length() > 0) && (blu >= 0) && (blu <= 255)) ? blu : 0;
        }

        rgb[0] = r;
        rgb[1] = g;
        rgb[2] = b;

        return rgb;
    }

    /**
     * Receives <code>stateChanged</code> events from this instance's <code>javax.swing.JSlider</code> components.
     * Values of the text fields are adjusted according to current color selection.
     *
     * @param e <code>java.awt.event.ChangeEvent</code>
     * @see     javax.swing.event.ChangeListener
     * @see     javax.swing.event.ChangeEvent
     * @since   1.0
     */
    public void stateChanged(ChangeEvent e)
    {
        int r, g, b;
        String redVal, grnVal, bluVal;
        redVal = grnVal = bluVal = "";
        int[] rgb = parseRGBfields();
        double[] rgbArr = ColorUtil.hexToFloatArray(ColorUtil.RGBtoHEX(rgb[0], rgb[1], rgb[2]));

        if(e.getSource().equals(sliderRed))
        {
            redVal = (isArithmeticConversion()) ? "" + rgbArr[0] : "" + sliderRed.getValue();
            fldRed.setText(redVal);
        }
        else if(e.getSource().equals(sliderGrn))
        {
            grnVal = (isArithmeticConversion()) ? "" + rgbArr[1] : "" + sliderGrn.getValue();
            fldGreen.setText(grnVal);
        }
        else if(e.getSource().equals(sliderBlu))
        {
            bluVal = (isArithmeticConversion()) ? "" + rgbArr[2] : "" + sliderBlu.getValue();
            fldBlue.setText(bluVal);
        }

        showColor(getRGBColor());
        showHexString();
        info.colorInformation(rgb[0], rgb[1], rgb[2], ColorUtil.RGBtoHEX(rgb[0], rgb[1], rgb[2]));
        info.repaint();
        showSafeColor();
    }

    /**
     * Sets RGB textfields and sliders to the correct values based on the value
     * of the <code>Hex</code> textfield.
     *
     * @see   javax.swing.JSlider
     * @since 1.0
     */
    public void setRGBcontrols()
    {
        String r, g, b;
        String[] hexs = new String[3];
        String hexValue = (ColorUtil.isValidHex(fldHex.getText())) ? fldHex.getText() : "000";
        double[] rgbArr = ColorUtil.hexToFloatArray(hexValue);
        int[] ints = ColorUtil.rgbFloatToUInt(rgbArr[0], rgbArr[1], rgbArr[2]);
        int len = fldHex.getText().length();

        if((len % 3) == 0)
        {
            hexs = ColorUtil.parseHex(fldHex.getText());
            r = hexs[0];
            g = hexs[1];
            b = hexs[2];

            if(isArithmeticConversion())
            {
                rgbArr = ColorUtil.hexToFloatArray(r + g + b);
                ints = ColorUtil.rgbFloatToUInt(rgbArr[0], rgbArr[1], rgbArr[2]);
                fldRed.setText(rgbArr[0] + "");
                fldGreen.setText(rgbArr[1] + "");
                fldBlue.setText(rgbArr[2] + "");
                sliderRed.setValue(ints[0]);
                sliderGrn.setValue(ints[1]);
                sliderBlu.setValue(ints[2]);
            }
            else
            {
                fldRed.setText(ColorUtil.HEXtoRGB(r) + "");
                fldGreen.setText(ColorUtil.HEXtoRGB(g) + "");
                fldBlue.setText(ColorUtil.HEXtoRGB(b) + "");
                sliderRed.setValue(ColorUtil.getRed(fldRed.getText()));
                sliderGrn.setValue(ColorUtil.getGreen(fldGreen.getText()));
                sliderBlu.setValue(ColorUtil.getBlue(fldBlue.getText()));
            }
        }
    }

    /**
     * Sets the current color selection.
     *
     * @param c <code>java.awt.Color</code> object of the current selected color.
     * @since   1.0
     */
    public void setCurrentColor(Color c)
    {
        int[] rgb;
        String hx;

        rgb = ColorUtil.parseAWTColor(c);
        hx = ColorUtil.RGBtoHEX(rgb[0], rgb[1], rgb[2]);

        setHexText(hx);
        setRGBcontrols();
        showColor(c);
        showHexString();
        info.colorInformation(rgb[0], rgb[1], rgb[2], hx);
        info.repaint();
        showSafeColor();
        //testResetState();
    }

    /**
     * Returns the nearest safe color based on the selected color.
     *
     * @param rgbArray <code>int</code> array of RGB values
     * @return         <code>java.awt.Color</code> object
     * @since          1.0
     */
    public Color getSafeColor(int[] rgbArray)
    {
        Color safeColor;
        int[] rgb = ColorUtil.nearestSafeColor(rgbArray);
        safeColor = ColorUtil.getNearestSafeColor(rgbArray);
        return safeColor;
    }

    /**
     * Returns the current color selection from the
     * <code>javax.swing.JColorChooser</code>.
     *
     * @param selection <code>int</code> value representing the
     *                  current chooser color value.
     * @see             javax.swing.JColorChooser
     * @since           1.0
     */
    public Color getChooserColor(int selection)
    {
        Color color;
        if(selection == RGB_VAL)
        {
            int[] rgb = parseRGBfields();
            color = new Color(rgb[0], rgb[1], rgb[2]);
        }
        else
        {
            String hexValue = fldHex.getText();
            int[] arr = ColorUtil.buildRGBarray(hexValue);
            int[] rgb = ColorUtil.nearestSafeColor(arr);
            color = new Color(rgb[0], rgb[1], rgb[2]);
        }
        return color;
    }

    /**
     * Opens a <code>javax.swing.JColorChooser</code> dialog.
     *
     * @param title     string for the title bar.
     * @param selection <code>int</code> value for selected color.
     * @see             javax.swing.JColorChooser
     * @see             javax.swing.JDialog
     * @see             javax.swing.JRootPane
     * @since           1.0
     */
    public void colorPicker(String title, final int selection)
    {
        c = getChooserColor(selection);
        final Color resetBg = getBgColor();
        final int sel = selection;
        final JColorChooser picker = new JColorChooser(c);
        final ColorSelectionModel csm = picker.getSelectionModel();
        csm.setSelectedColor(c);
        JDialog dialog = JColorChooser.createDialog(host, title, true, picker,
        new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                newColor = picker.getColor(); ;
                colorSelection(sel);
            }
        }, null);
        dialog.getRootPane().setWindowDecorationStyle(JRootPane.PLAIN_DIALOG);
        dialog.setVisible(true);
    }

    private void colorSelection(int selection)
    {
        final Color reset = colorSwatch.getBackground();
        switch(selection)
        {
            case 1:
                setCurrentColor(newColor);
                host.repaint();
                break;
            case 2:
                setCurrentColor(newColor);
                host.repaint();
                break;
            case 3:
                setBgColor(newColor);
                cp.setBackground(newColor);
                restart();
                setCurrentColor(reset);
                break;
        }
    }

    /**
     * Opens the palette windows. The palette windows calls the <code>dispose</code>
     * method of <code>javax.swing.JFrame</code> instead of <code>System.exit</code>
     * during window close events so that the parent window is not destroyed.
     *
     * @param title   string for the title bar.
     * @param palette <code>int</code> value representing the palette type.
     * @see           java.awt.event.WindowAdapter
     * @see           java.awt.event.WindowEvent
     * @see           ColorModelView
     * @since         1.0
     */
    public void openColorPalette(String title, int palette)
    {
        // Ugly LookAndFeel hack for problem with Metal LAF.
        ColorModelView tmp = new ColorModelView(this, host, title, palette);
        tmp.setDefaultLookAndFeelDecorated(true);
        tmp.pack();
        tmp = null; // End ugliness;
        // Begin normalcy
        final ColorModelView cmv = new ColorModelView(this, host, title, palette);
        cmv.setDefaultLookAndFeelDecorated(true);
        cmv.setResizable(false);
        cmv.pack();
        Dimension center = WindowUtil.centerWindow(host, cmv);
        cmv.setLocation(center.width, center.height);
        cmv.setVisible(true);
        // Handle window close event
        cmv.addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                cmv.setVisible(false);
                cmv.dispose();
            }
        });
        // Dispose of deactivated windows
        cmv.addWindowListener(new WindowAdapter()
        {
            public void windowDeactivated(WindowEvent e)
            {
                cmv.setVisible(false);
                cmv.dispose();
            }
        });
    }

    /**
     * Sets the text value of the <code>Hex</code> text field.
     *
     * @see   javax.swing.JTextField
     * @since 1.0
     */
    public void setHexText(String str)
    {
        fldHex.setText(str);
    }

    /**
    * Returns the insets for <i>GridBag</i> padding of components in this window.
    *
    * @return <code>java.awt.Insets</code> object
    * @see    java.awt.Insets
    * @since  1.0
    */
    public Insets getInsets()
    {
        return new Insets(20, 10, 10, 10);
    }

    /************************************************************
     msgDialog optionType settings (not implemented yet)
     optionType::
     DEFAULT_OPTION, YES_NO_OPTION, YES_NO_CANCEL_OPTION,
     OK_CANCEL_OPTION
    *************************************************************/

    /**
     * Handles error, warning and information messages by opening a dialog.
     * <p>
     * The <code>msgType</code> parameter of the <code>msgDialog</code> can
     * accept the following parameters,
     * <code>
     * <ul>
     *   <li>JOptionPane.ERROR_MESSAGE
     *   <li>JOptionPane.INFORMATION_MESSAGE
     *   <li>JOptionPane.WARNING_MESSAGE
     *   <li>JOptionPane.QUESTION_MESSAGE
     *   <li>JOptionPane.PLAIN_MESSAGE
     * </ul>
     * </code>
     * @param parent  parent component for the dialog.
     * @param msg     message to disply in the content pane.
     * @param title   title bar text.
     * @param msgType defines the style of the message.
     * @since         1.0
     */
    public void msgDialog(Component parent, Object msg, String title, int msgType)
    {
        JOptionPane.showMessageDialog(parent, msg, title, msgType);
    }

    /**
     * Displays the <code>About</code> object in a bordless window - called from <code>Help</code> menu.
     * The <code>About</code> class is called statically since it is a stand-alone application
     * containing a <code>main</code> method.
     *
     * @since 1.0
     */
    public void showAboutBox() throws IOException
    {
        String[] args = {};
        About.setHost(host);
        About.setController(this);
        About.setBoxSize(298, 203);
        About.setDefaultResourcePath(DEF_RES_PATH);
        About.setBackgroundImageString(App.ABOUT_IMG);
        About.setTitleBar(App.TITLE_APP);
        About.main(args);
    }

    //***************************************
    //           Validation Code
    //***************************************

    /**
     * Sets the maximum number of characters that are allowed to be entered into the textfields.
     *
     * @param e <code>java.awt.event.KeyEvent</code> for text field validation.
     * @see     java.awt.event.KeyEvent
     * @see     java.lang.Character
     * @since   1.0
     */
    protected void validateTextField(KeyEvent e)
    {
        char c = e.getKeyChar();
        for(int i = 0; i < controlsLength; ++i)
        {
            if((e.getSource().equals(jFields[i])))
            {
                int len = jFields[i].getText().length();
                if(len < TXT_MAXLENGTH)
                {
                  return;
                }
                else
                {
                    if((c==KeyEvent.VK_BACK_SPACE) ||
                       (c==KeyEvent.VK_DELETE) ||
                       (c==KeyEvent.VK_ENTER) ||
                       (c==KeyEvent.VK_TAB) || e.isActionKey())
                    {
                       return;
                    }
                    else
                    {
                        e.consume();
                    }
                }
            }
        }
    }

    /**
     * Completes validation for the RGB text fields. Prevents illegal values form being entered.
     *
     * @param fe  <code>java.awt.event.FocusEvent</code> for text field validation.
     * @param txt <code>javax.swing.JTextField</code> reference for validation.
     * @see       java.awt.event.FocusEvent
     * @since     1.0
     */
    protected void finalValidation(FocusEvent fe, JTextField txt)
    {
        for(int i = 0; i < 3; ++i)
        {
            if((fe.getSource().equals(jFields[i])))
            {
                if(!isArithmeticConversion())
                {
                    try
                    {
                        num = Integer.parseInt(jFields[i].getText());
                        if((jFields[i].getText().length() != 0))
                        {
                            jSliders[i].setValue(num); // set this slider to current value
                        }
                    }
                    catch(NumberFormatException nfe)
                    {
                        jFields[i].setText("0");
                        jSliders[i].setValue(0);
                        applyAction();
                    }
                }
                else
                {
                    try
                    {
                        applyAction();
                    }
                    catch(NumberFormatException nfe)
                    {
                        jFields[i].setText("0");
                        jSliders[i].setValue(0);
                        applyAction();
                    }
                }
            } // End If
        } // End For
    } // End function

} // End class